@@ -1,7 +1,6 @@
 #!/usr/bin/env python3
 
 import argparse
-import datetime
 import decimal
 import multiprocessing
 import os
@@ -141,7 +140,7 @@ def parse_arguments():
             dest='no_stars',
             required=False,
             action='store_true',
-            help='Suppress marking of suggestive classifications.')
+            help='Suppress marking of suggestive taxonomic assignments.')
     optional.add_argument(
             '--force',
             dest='force',
@@ -314,7 +313,7 @@ def import_bins(bin_folder, bin_suffix, log_file, quiet):
     if len(bin2contigs) == 1:
         message = '1 bin found!'
     else:
-        message = '{0} bins found!'.format(len(bin2contigs))
+        message = '{0:,d} bins found!'.format(len(bin2contigs))
     shared.give_user_feedback(message, log_file, quiet)
 
     return (bin2contigs, contig_names)
@@ -570,7 +569,8 @@ def run():
 
     with open(args.bin2classification_output_file, 'w') as outf1, open(args.ORF2LCA_output_file, 'w') as outf2:
         outf1.write('# bin\tclassification\treason\tlineage\tlineage scores\n')
-        outf2.write('# ORF\tbin\tlineage\tbit-score\n')
+
+        outf2.write('# ORF\tbin\tnumber of hits\tlineage\ttop bit-score\n')
         
         for bin_ in sorted(bin2contigs):
             LCAs_ORFs = []
@@ -586,6 +586,8 @@ def run():
 
                         continue
 
+                    n_hits = len(ORF2hits[ORF])
+
                     (taxid,
                             top_bitscore) = tax.find_LCA_for_ORF(
                                     ORF2hits[ORF],
@@ -593,8 +595,8 @@ def run():
                                     taxid2parent)
                      
                     if taxid.startswith('no taxid found'):
-                        outf2.write('{0}\t{1}\t{2}\t{3}\n'.format(
-                            ORF, bin_, taxid, top_bitscore))
+                        outf2.write('{0}\t{1}\t{2}\t{3}\t{4}\n'.format(
+                            ORF, bin_, n_hits, taxid, top_bitscore))
                     else:
                         lineage = tax.find_lineage(taxid, taxid2parent)
 
@@ -602,14 +604,18 @@ def run():
                             lineage = tax.star_lineage(
                                     lineage, taxids_with_multiple_offspring)
 
-                        outf2.write('{0}\t{1}\t{2}\t{3}\n'.format(
-                            ORF, bin_, ';'.join(lineage[::-1]), top_bitscore))
+                        outf2.write('{0}\t{1}\t{2}\t{3}\t{4}\n'.format(
+                            ORF,
+                            bin_,
+                            n_hits,
+                            ';'.join(lineage[::-1]),
+                            top_bitscore))
                                        
                     LCAs_ORFs.append((taxid, top_bitscore),)
                     
             if len(LCAs_ORFs) == 0:
-                outf1.write('{0}\tunclassified\tno hits to database\n'.format(
-                    bin_))
+                outf1.write('{0}\tno taxid assigned\tno hits to database\n'
+                        ''.format(bin_))
 
                 continue
 
@@ -619,14 +625,14 @@ def run():
                             LCAs_ORFs, taxid2parent, args.f)
 
             if lineages == 'no ORFs with taxids found.':
-                outf1.write('{0}\tunclassified\t'
+                outf1.write('{0}\tno taxid assigned\t'
                         'hits not found in taxonomy files\n'.format(bin_))
 
                 continue
 
             if lineages == 'no lineage whitelisted.':
                 outf1.write(
-                        '{0}\tunclassified\t'
+                        '{0}\tno taxid assigned\t'
                         'no lineage reached minimum bit-score support\n'
                         ''.format(bin_))
 
@@ -650,7 +656,7 @@ def run():
                     # There is only one classification.
                     outf1.write(
                             '{0}\t'
-                            'classified\t'
+                            'taxid assigned\t'
                             'based on {1}/{2} ORFs\t'
                             '{3}\t'
                             '{4}\n'.format(
@@ -663,7 +669,7 @@ def run():
                     # There are multiple classifications.
                     outf1.write(
                             '{0}\t'
-                            'classified ({1}/{2})\t'
+                            'taxid assigned ({1}/{2})\t'
                             'based on {3}/{4} ORFs\t'
                             '{5}\t'
                             '{6}\n'.format(
@@ -676,10 +682,8 @@ def run():
                                 ';'.join(scores[::-1])))
                                    
     message = ('\n-----------------\n\n'
-               '[{0}] BAT is done! {1}/{2} bins classified.'.format(
-                   datetime.datetime.now(),
-                   n_classified_bins,
-                   len(bin2contigs)))
+            '{0} BAT is done! {1:,d}/{2:,d} bins have taxonomy assigned.'
+            ''.format(shared.timestamp(), n_classified_bins, len(bin2contigs)))
     shared.give_user_feedback(message, args.log_file, args.quiet,
             show_time=False)
   
